# -*- coding: utf-8 -*-

from .design_pattern import singleton

@singleton()
class ItertoolsUtilsClass(object):
    def split_seqs_by_size(self, seqs1, size1, inspect=False):
        """
        Combinations of split seqs by special size.

        Breadth-first version, see examples at bottom.

        Thanks @fuchaoqun for the recursion idea.
        """

        if inspect: print "[split_seqs_by_size]", seqs1, size1
        seqs_len = len(seqs1)
        assert seqs_len >= size1, "seqs1 len should greater or equal than size1."
        assert size1 > 0, "size1 should greater than zero.."
        final_result = []

        def func(seqs2, size2, pre_result2=[]):
            if inspect: print; print "pre_result2", pre_result2

            if size2 == 1: return final_result.append(pre_result2 + [seqs2])

            for idx3 in range(len(seqs2) - size2 + 1):
                idx4 = idx3 + 1
                if inspect: print ["size2", size2, "idx4", idx4]
                if inspect: print "[loop seqs2]", seqs2[:idx4], seqs2[idx4:]

                # generate a new result object
                current_result = pre_result2 + [seqs2[:idx4]]

                if len(current_result) == size1:
                    final_result.append(current_result)
                else:
                    func(seqs2[idx3+1:], size2-1, current_result)
        func(seqs1, size1)

        if inspect: print "\n"*2
        return final_result

ItertoolsUtils = ItertoolsUtilsClass()

"""
Iterator space analysis.

seqs_len,   chunk_size,   index_times
       5,            5,             1
       5,            4,             2
       5,            3,             3
       5,            2,             4
       5,            1,             1

       4,            4,             1
       4,            3,             2
       4,            2,             3
       4,            4,             1
"""



# -*- coding: utf-8 -*-

# 引入XML解析库
# xml.etree.ElementTree解析XML遇到 fontFamily="\u5b8b\u4f53" 这种属性时会无效
# 详情见 http://lxml.de/parsing.html#python-unicode-strings 描述

from itertools import chain
from .design_pattern import singleton, cached_property

@singleton()
class TextArrayXMLParserClass(object):
    """
    XML题目内容parse，见fullcontent.question.choice里的参考数据说明

    XPATH结构是 TextFlow{} -> p{} -> span{}.text
    """

    def parse(self, xml1):
        str_list = [[span1.text for span1 in p1.getchildren() if span1.text]
            # '\x1f' cause xml parser error
            for p1 in self.etree.XML(xml1.encode("UTF-8").replace('\x1f', '')).getchildren()]
        str_list = list(chain.from_iterable(str_list))

        return ' '.join(str_list)

    @cached_property
    def etree(self):
        # 'lxml >= 3.3.5',
        from lxml import etree
        return etree

TextArrayXMLParser = TextArrayXMLParserClass()



# -*- coding: utf-8 -*-

import os
import progressbar as pb


class ProcessNotifier(object):
    """
    Usage:

    for b1 in ProcessNotifier(Book).generator():
        # do b1
    """

    def __init__(self, scope, per, msg):
        """ TypeError: __init__() should return None, not 'generator' """
        from .object_utils import set_default_value

        scope_repr = unicode(repr(type(scope)))

        self.iterator = set_default_value(
              [lambda : scope.iteritems(), lambda : scope.iterator(), lambda: iter(scope), ], \
              scope_repr + u" should be iteratable!")

        self.current_pid = os.getpid()
        self.per = per
        self.msg = msg

        # 兼容 list, dict, mongomock
        # 判断这个对象的属性方法来觉得用len还是count，不能覆盖所有情况，所以这里直接暴力解决。
        self.total_count = set_default_value( \
              [ lambda : len(scope), lambda : scope.count()], \
              scope_repr + u" should be counted!")


    def generator(self):
        """ Use a Generator to print the inner status when iterate a scope."""
        base_str = u"[pid %i] %s processing %i records " % (self.current_pid, self.msg, self.total_count)

        process_count = 0

        widgets = ['  ', base_str.encode("UTF-8"), pb.Percentage(), ' ', pb.Bar(),
                   ' ', ItemProcessSpeed()]

        if self.total_count:
            self.pbar = pb.ProgressBar(widgets=widgets, maxval=self.total_count).start()

        for record in self.iterator:
            process_count += 1
            # Fix grow up when iterator
            if process_count > self.total_count: process_count = self.total_count
            if self.total_count: self.pbar.update(process_count)
            yield record
        print


class ItemProcessSpeed(pb.ProgressBarWidget):
    def __init__(self):
        self.fmt = '%6.2f items/s'
    def update(self, pbar):
        if pbar.seconds_elapsed < 2e-6:#== 0:
            bps = 0.0
        else:
            bps = float(pbar.currval) / pbar.seconds_elapsed
        return self.fmt % bps


def process_notifier(scope, per=1000, msg=u"",):
    assert isinstance(msg, unicode)
    return ProcessNotifier(scope, per, msg).generator()



# -*- coding: utf-8 -*-

from .design_pattern import cached_property, singleton

_ = u""

class UnicodeConvert(object):
    """
    汉字处理的工具:
    copied from https://github.com/oldhu/micolog-oldhu/blob/master/app/gbtools.py
    判断unicode是否是汉字，数字，英文，或者其他字符。
    全角符号转半角符号。
    """

    def is_chinese(self, uchar):
        """判断一个unicode是否是汉字"""
        assert isinstance(uchar, unicode)
        return uchar >= u'\u4e00' and uchar <= u'\u9fa5'

    def is_number(self, uchar):
        """判断一个unicode是否是数字"""
        assert isinstance(uchar, unicode)
        return uchar >= u'\u0030' and uchar <= u'\u0039'

    def is_alphabet(self, uchar):
        """判断一个unicode是否是英文字母"""
        assert isinstance(uchar, unicode)
        return (uchar >= u'\u0041' and uchar <= u'\u005a') \
                or (uchar >= u'\u0061' and uchar <= u'\u007a')

    def is_other(self, uchar):
        """判断是否非汉字，数字和英文字符"""
        assert isinstance(uchar, unicode)
        return not (UnicodeUtils.is_chinese(uchar) or UnicodeUtils.is_number(uchar) \
                or UnicodeUtils.is_alphabet(uchar))

    def B2Q(self, uchar):
        """半角转全角"""
        assert isinstance(uchar, unicode)
        inside_code = ord(uchar)
        if inside_code < 0x0020 or inside_code > 0x7e:      #不是半角字符就返回原来的字符
                return uchar
        if inside_code == 0x0020: #除了空格其他的全角半角的公式为:半角=全角-0xfee0
                inside_code = 0x3000
        else:
                inside_code += 0xfee0
        return unichr(inside_code)

    def is_Q(self, uchar):
        return uchar != Q2B(uchar)

    def Q2B(self, uchar):
        """全角转半角"""
        assert isinstance(uchar, unicode)
        inside_code = ord(uchar)
        if inside_code == 0x3000:
                inside_code = 0x0020
        else:
                inside_code -= 0xfee0
        if inside_code < 0x0020 or inside_code > 0x7e:      #转完之后不是半角字符返回原来的字符
                return uchar
        return unichr(inside_code)

    def stringQ2B(self, ustring, convert_strs={}):
        """把字符串全角转半角"""
        default_Q2B_convert = {
                                  unicode("…", "UTF-8") : u"...",
                                  u"′"                  : u"'",
                              }
        convert_strs = convert_strs or default_Q2B_convert

        assert isinstance(ustring, unicode)
        result = [UnicodeUtils.Q2B(uchar) for uchar in ustring]
        result = [(((uchar in convert_strs) and convert_strs[uchar]) or uchar) for uchar in result]
        return _.join(result)

    def uniform(self, ustring):
        """格式化字符串，完成全角转半角，大写转小写的工作"""
        return UnicodeUtils.stringQ2B(ustring).lower()

    def string2List(self, ustring):
        """将ustring按照中文，字母，数字分开"""
        retList=[]
        utmp=[]
        for uchar in ustring:
                if UnicodeUtils.is_other(uchar):
                        if len(utmp)==0:
                                continue
                        else:
                                retList.append(_.join(utmp))
                                utmp=[]
                else:
                        utmp.append(uchar)
        if len(utmp)!=0:
                retList.append(_.join(utmp))
        return retList


class UnicodeWidth(object):
    """ 处理unicode在终端下的显示宽度 """

    def ljust(self, str1, width, fillchar=' '):
        return self.just_str(str1, 'ljust', width, fillchar)

    def rjust(self, str1, width, fillchar=' '):
        return self.just_str(str1, 'rjust', width, fillchar)

    ######### private functions
    @cached_property
    def just_str(self):
        """
        兼容中文和全角字符的对齐打印。Compact with chinese ajust.
        """
        # `pip install urwid`
        from urwid import is_wide_char
        def func(str1, method, width, fillchar):
            if isinstance(str1, str): str1 = unicode(str1, "UTF-8")
            assert isinstance(str1, unicode)

            two_width_count = len([s1 for s1 in str1 if is_wide_char(s1, 0)])
            return getattr(str1, method)(width - two_width_count, fillchar)
        return func

class UnicodeFile(object):
    def read(self, filename):
        return open(filename).read().decode("UTF-8")


@singleton()
class UnicodeClass(UnicodeConvert, UnicodeWidth, UnicodeFile):
    pass

UnicodeUtils = UnicodeClass()
Unicode = UnicodeUtils



# -*- coding: utf-8 -*-

import re
from .design_pattern import singleton, cached_property

@singleton()
class RegexpUtils(object):

    @cached_property
    def alphabet(self): return re.compile("[a-z]", re.IGNORECASE)

    @cached_property
    def word(self): return re.compile("^[a-z]+$", re.IGNORECASE)

    @cached_property
    def upper(self): return re.compile("[A-Z]")

    @cached_property
    def object_id(self): return re.compile("^[a-z0-9]{24}$")

    @cached_property
    def special_chars(self):
        return re.compile(
                           r"""
                            ^X√$|^FT$| # 选择题答案的hash特别处理
                            \ |,|\.|\?|\n|\'|!| # 正常字符
                            ’|—|？|;|。|，|：|）|（|．|！|、|“|”|\"|=|‘|′|\/|　|\\|＿|…|\+|\–|Ⅰ|Ⅱ|`| |@|－|；|｀|＇|•|∶ # 特殊字符
                            """,
                        re.VERBOSE)

"""
`cat txt/duplicated_fullcontent_result_random500_20140507_1053.txt | grep -v 当前平均 | grep -v 字符串相 | grep -v 个题目 | grep -v self.time | grep text`.split("\n").map {|line| line.split(/text[12]:  /)[1] }.join().split(//).uniq.reject {|c1| c1.match(/[\u4E00-\u9FA5]+/) }.reject {|c1| c1.match(/[a-z]/i) }.reject {|c1| c1.match(/[0-9]/) }.join("|")
# "’|—|？|;|。|，|：|）|（|．|！|、|“|”|\"|=|‘|′|/|　|\\|＿|…|+|–|Ⅰ|Ⅱ|`| |@|－|；|｀|＇|•|∶|４|２|１|ɑ|→|&|￥|´|√|×|ə|ɜ|ː|<|>|①|②|③|④|¥||"
"""

regexp = RegexpUtils()



# -*- coding: utf-8 -*-

import math

def calculate_entropy(feature_with_count_dict):
    assert hasattr(feature_with_count_dict, 'iteritems')
    assert isinstance(feature_with_count_dict.iteritems().next()[1], int)

    feature_count_sum = float(sum(feature_with_count_dict.values()))

    entropy = 0.0
    for f1, c1 in feature_with_count_dict.iteritems():
        p_ij = c1 / feature_count_sum
        entropy += p_ij * math.log(p_ij)
    return - entropy
